from typing import Any, Dict, Type, TypeVar, Union

import attr

from ..models.api_value_composition_array import ApiValueCompositionArray
from ..types import UNSET, Unset

T = TypeVar("T", bound="ApiOutputCalculationResultPhaseComposition")


@attr.s(auto_attribs=True)
class ApiOutputCalculationResultPhaseComposition:
    """Holds composition information for a phase.
    
    Attributes
    ----------
    composition_units : str
        Units of the composition.
    molar_mass_units : str
        Units of the molar mass.
    composition : ApiValueCompositionArray
        Composition of the phase.
    """

    composition_units: Union[Unset, None, str] = UNSET
    molar_mass_units: Union[Unset, None, str] = UNSET
    composition: Union[Unset, ApiValueCompositionArray] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        """Return a dictionary representation of this object."""
        composition_units = self.composition_units
        molar_mass_units = self.molar_mass_units
        composition: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.composition, Unset):
            composition = self.composition.to_dict()

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if not isinstance(composition_units, Unset):
            field_dict["compositionUnits"] = composition_units
        if not isinstance(molar_mass_units, Unset):
            field_dict["molarMassUnits"] = molar_mass_units
        if not isinstance(composition, Unset):
            field_dict["composition"] = composition

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        """Create an instance of this class from a dictionary."""
        d = src_dict.copy()
        composition_units = d.pop("compositionUnits", UNSET)

        molar_mass_units = d.pop("molarMassUnits", UNSET)

        _composition = d.pop("composition", UNSET)
        composition: Union[Unset, ApiValueCompositionArray]
        if isinstance(_composition, Unset):
            composition = UNSET
        else:
            composition = ApiValueCompositionArray.from_dict(_composition)

        api_output_calculation_result_phase_composition = cls(
            composition_units=composition_units,
            molar_mass_units=molar_mass_units,
            composition=composition,
        )

        return api_output_calculation_result_phase_composition

    def __str__(self) -> str:
        """
        Returns a string representation of the phase composition.
        """
        units = self.composition_units if not isinstance(self.composition_units, Unset) else "N/A"
        mm_units = self.molar_mass_units if not isinstance(self.molar_mass_units, Unset) else "N/A"
        lines = [f"Units: {units}, Molar Mass Units: {mm_units}"]

        if not isinstance(self.composition, Unset) and self.composition is not None:
            lines.append(str(self.composition))
        else:
            lines.append("Composition: N/A")

        return "\n".join(lines)



from typing import Any, Dict, List, Type, TypeVar, Union

import attr

from ..models.calculation_composition import CalculationComposition
from ..models.api_fluid import ApiFluid
from ..types import UNSET, Unset

T = TypeVar("T", bound="EosPropertiesTPnCalculationInput")


@attr.s(auto_attribs=True)
class EosPropertiesTPnCalculationInput:
    """Input to EoS TPn property calculation. 

    Attributes
    ----------
    access_key : str
        Access key for the API. This is used to authenticate the user.
    components : List[CalculationComposition]
        List of components in the mixture.
    units : str
        Units for the calculation.
    volumetype : str
        Volume root to use. Allowed values are: `{'Auto', 'Liquid', 'Vapor'}`
    fluidid : str
        Id of fluid on webserver. Must be defined if no fluid given in fluid argument
    fluid : ApiFluid
        Fluid information
    temperature : float
        Temperature in units given in `units` argument.
    pressure : float
        Pressure in units given in `units` argument.
    """

    access_key: str
    components: List[CalculationComposition]
    units: str
    volumetype: str # Volume root to use. Allowed values are: Auto, Liquid, Vapor
    fluidid: Union[Unset, str] = UNSET #Id of fluid on webserver. Must be defined if no fluid given in fluid argument
    fluid: Union[Unset, ApiFluid] = UNSET #Fluid information
    temperature: Union[Unset, float] = UNSET
    pressure: Union[Unset, float] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        """Return a dictionary representation of this object."""
        access_key = self.access_key
        components = []
        for components_item_data in self.components:
            components_item = components_item_data.to_dict()
            components.append(components_item)

        units = self.units
        volumetype = self.volumetype
        
        temperature = self.temperature
        pressure = self.pressure

        fluidid: Union[Unset, str] = UNSET
        if not isinstance(self.fluidid, Unset):
            fluid = self.fluidid

        fluid: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.fluid, Unset):
            fluid = self.fluid.to_dict()

        field_dict: Dict[str, Any] = {}
        field_dict.update(
            {
                "accessKey": access_key,
                "components": components
            }
        )
        if not isinstance(units, Unset):
            field_dict["units"] = units
        if not isinstance(volumetype, Unset):
            field_dict["volumetype"] = volumetype
        if not isinstance(fluidid, Unset):
            field_dict["fluidId"] = fluidid
        if not isinstance(fluid, Unset):
            field_dict["fluid"] = fluid
        if not isinstance(temperature, Unset):
            field_dict["temperature"] = temperature
        if not isinstance(pressure, Unset):
            field_dict["pressure"] = pressure

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        """Create an instance of `EosPropertiesTPnCalculationInput` from a dict representation."""
        d = src_dict.copy()

        access_key = d.pop("accessKey")

        components = []
        _components = d.pop("components")
        for components_item_data in _components:
            components_item = CalculationComposition.from_dict(components_item_data)

            components.append(components_item)

        _fluidid = d.pop("fluidId", UNSET)
        fluidid: Union[Unset, str]
        if isinstance(_fluidid, Unset):
            fluidid = UNSET
        else:
            fluidid = _fluidid

        _fluid = d.pop("fluid", UNSET)
        fluid: Union[Unset, ApiFluid]
        if isinstance(_fluid, Unset):
            fluid = UNSET
        else:
            fluid = ApiFluid.from_dict(_fluid)

        units = d.pop("units", UNSET)
        volumetype = d.pop("volumetype", UNSET)
        
        temperature = d.pop("temperature", UNSET)

        pressure = d.pop("pressure", UNSET)

        eospropertiestpn_calculation_input = cls(
            access_key=access_key,
            components=components,
            fluidid=fluidid,
            fluid=fluid,
            units=units,
            volumetype=volumetype,
            temperature=temperature,
            pressure=pressure,
        )

        return eospropertiestpn_calculation_input

    def __str__(self) -> str:
        """
        Returns a readable string representation of the TPn calculation input.
        """
        lines = ["EosPropertiesTPnCalculationInput:"]

        lines.append(f"  Fluid ID: {self.fluidid if not isinstance(self.fluidid, Unset) else 'N/A'}")
        lines.append(f"  Units: {self.units}")
        lines.append(f"  Temperature: {self.temperature if not isinstance(self.temperature, Unset) else 'N/A'}")
        lines.append(f"  Pressure: {self.pressure if not isinstance(self.pressure, Unset) else 'N/A'}")
        lines.append(f"  Volume Type: {self.volumetype}")

        # Components
        if self.components:
            lines.append("  Components:")
            for comp in self.components:
                lines.append("    " + str(comp).replace("\n", "\n    "))
        else:
            lines.append("  Components: None")

        # Fluid
        if not isinstance(self.fluid, Unset) and self.fluid:
            lines.append("  Fluid:")
            lines.append("    " + str(self.fluid).replace("\n", "\n    "))
        else:
            lines.append("  Fluid: N/A")

        return "\n".join(lines)



from typing import Any, Dict, Type, TypeVar, Union

import attr

from ..models.api_output_calculation_result_point import ApiOutputCalculationResultPoint
from ..models.api_profile_information import ApiProfileInformation
from ..models.exception_info import ExceptionInfo
from ..types import UNSET, Unset

T = TypeVar("T", bound="FlashCalculationResult")


@attr.s(auto_attribs=True)
class FlashCalculationResult:
    """Holds calculation result for a point. 
    
    Attributes
    ----------
    success : bool
        Indicates if the calculation was successful.
    calculation_id : str
        Unique identifier for the calculation.
    exception_info : ExceptionInfo
        Information about any exceptions that occurred during the calculation.
    profiling : ApiProfileInformation
        Information about the profiling of the calculation.
    point : ApiOutputCalculationResultPoint
        Calculation result for the point.
    """

    success: Union[Unset, bool] = UNSET
    calculation_id: Union[Unset, None, str] = UNSET
    exception_info: Union[Unset, ExceptionInfo] = UNSET
    profiling: Union[Unset, ApiProfileInformation] = UNSET
    point: Union[Unset, ApiOutputCalculationResultPoint] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        """Dictionary representation of the object."""
        success = self.success
        calculation_id = self.calculation_id
        exception_info: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.exception_info, Unset):
            exception_info = self.exception_info.to_dict()

        profiling: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.profiling, Unset):
            profiling = self.profiling.to_dict()

        point: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.point, Unset):
            point = self.point.to_dict()

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if not isinstance(success, Unset):
            field_dict["success"] = success
        if not isinstance(calculation_id, Unset):
            field_dict["calculationId"] = calculation_id
        if not isinstance(exception_info, Unset):
            field_dict["exceptionInfo"] = exception_info
        if not isinstance(profiling, Unset):
            field_dict["profiling"] = profiling
        if not isinstance(point, Unset):
            field_dict["point"] = point

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        """Create an instance of `FlashCalculationResult` from a dict representation."""
        d = src_dict.copy()
        success = d.pop("success", UNSET)

        calculation_id = d.pop("calculationId", UNSET)

        _exception_info = d.pop("exceptionInfo", UNSET)
        exception_info: Union[Unset, ExceptionInfo]
        if isinstance(_exception_info, Unset):
            exception_info = UNSET
        else:
            exception_info = ExceptionInfo.from_dict(_exception_info)

        _profiling = d.pop("profiling", UNSET)
        profiling: Union[Unset, ApiProfileInformation]
        if isinstance(_profiling, Unset):
            profiling = UNSET
        else:
            profiling = ApiProfileInformation.from_dict(_profiling)

        _point = d.pop("point", UNSET)
        point: Union[Unset, ApiOutputCalculationResultPoint]
        if isinstance(_point, Unset):
            point = UNSET
        else:
            point = ApiOutputCalculationResultPoint.from_dict(_point)

        flash_calculation_result = cls(
            success=success,
            calculation_id=calculation_id,
            exception_info=exception_info,
            profiling=profiling,
            point=point,
        )

        return flash_calculation_result

    def __str__(self) -> str:
        """
        Returns a string representation of the FlashCalculationResult instance.
        
        The string includes:
        - Success (True/False),
        - Calculation ID (if available; otherwise, "N/A"),
        - Exception Info (if available; otherwise, "N/A"),
        - Profiling (if available; otherwise, "N/A"),
        - Point (if available; otherwise, "N/A").

        Returns
        -------
        str
            A string representation of the instance.
        """
        parts = ["\n"]
        parts.append("FlashCalculationResult:")
        # Success and Calculation ID
        success_str = self.success if not isinstance(self.success, Unset) else "N/A"
        calc_id_str = self.calculation_id if not isinstance(self.calculation_id, Unset) else "N/A"
        parts.append(f"  Success: {success_str}")
        parts.append(f"  Calculation ID: {calc_id_str}")
        
        # Exception Info
        if not isinstance(self.exception_info, Unset) and self.exception_info is not None:
            parts.append(f"  Exception Info: {self.exception_info}")
        else:
            parts.append("  Exception Info: N/A")
        
        # Profiling
        if not isinstance(self.profiling, Unset) and self.profiling is not None:
            parts.append("  Profiling:")
            # Call the __str__ method of profiling
            parts.append("    " + str(self.profiling).replace("\n", "\n    "))
        else:
            parts.append("  Profiling: N/A")
        
        # Point
        if not isinstance(self.point, Unset) and self.point is not None:
            parts.append("  Point:")
            # Call the __str__ method of point
            parts.append("    " + str(self.point).replace("\n", "\n    "))
        else:
            parts.append("  Point: N/A")
        
        return "\n".join(parts)



from typing import Any, Dict, Type, TypeVar, Union

import attr

from ..models.api_output_eospropertiestpn_ideal import ApiOutputEosPropertiesTPnResultIdeal
from ..models.api_output_eospropertiestpn_residual import ApiOutputEosPropertiesTPnResultResidual
from ..models.api_value_pressure import ApiValuePressure
from ..models.api_value_temperature import ApiValueTemperature
from ..models.api_value_volume import ApiValueVolume
from ..types import UNSET, Unset

T = TypeVar("T", bound="ApiOutputEosPropertiesTPnResult")


@attr.s(auto_attribs=True)
class ApiOutputEosPropertiesTPnResult:
    """Holds result for a point. This is stored in the `EosPropertiesTPnCalculationResult.point` attribute.
    
    Attributes
    ----------
    temperature : ApiValueTemperature
        Temperature of the phase.
    pressure : ApiValuePressure
        Pressure of the phase.
    volume : ApiValueVolume
        Volume of the phase.
    attractive_pressure : ApiValuePressure
        Attractive pressure of the phase.
    repulsive_pressure : ApiValuePressure
        Repulsive pressure of the phase.
    residual : ApiOutputEosPropertiesTPnResultResidual
        Residual thermodynamic properties of the phase.
    ideal : ApiOutputEosPropertiesTPnResultIdeal
        Ideal gas thermodynamic properties of the phase.
    """

    temperature: Union[Unset, ApiValueTemperature] = UNSET
    pressure: Union[Unset, ApiValuePressure] = UNSET
    volume: Union[Unset, ApiValueVolume] = UNSET
    attractive_pressure: Union[Unset, ApiValuePressure] = UNSET
    repulsive_pressure: Union[Unset, ApiValuePressure] = UNSET
    residual: Union[Unset, ApiOutputEosPropertiesTPnResultResidual] = UNSET
    ideal: Union[Unset, ApiOutputEosPropertiesTPnResultIdeal] = UNSET

    FIELD_MAP = {
        "temperature": "temperature",
        "pressure": "pressure",
        "volume": "volume",
        "attractive_pressure": "attractivePressure",
        "repulsive_pressure": "repulsivePressure",
        "residual": "residual",
        "ideal": "ideal",
    }

    def to_dict(self) -> Dict[str, Any]:
        """Dump an instance of `ApiOutputEosPropertiesTPnResult` to a dict."""
        result: Dict[str, Any] = {}
        for attr_name, api_key in self.FIELD_MAP.items():
            val = getattr(self, attr_name)
            if isinstance(val, Unset):
                continue
            result[api_key] = val.to_dict() if hasattr(val, "to_dict") else val
        return result

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        """Create an instance of `ApiOutputEosPropertiesTPnResult` from a dict."""
        d = src_dict.copy()
        kwargs = {}

        for field, api_key in cls.FIELD_MAP.items():
            value = d.pop(api_key, UNSET)

            # custom deserialization for known nested models
            if field == "temperature" and not isinstance(value, Unset):
                value = ApiValueTemperature.from_dict(value)
            elif field == "pressure" and not isinstance(value, Unset):
                value = ApiValuePressure.from_dict(value)
            elif field == "volume" and not isinstance(value, Unset):
                value = ApiValueVolume.from_dict(value)
            elif field == "attractive_pressure" and not isinstance(value, Unset):
                value = ApiValuePressure.from_dict(value)
            elif field == "repulsive_pressure" and not isinstance(value, Unset):
                value = ApiValuePressure.from_dict(value)
            elif field == "residual" and not isinstance(value, Unset):
                value = ApiOutputEosPropertiesTPnResultResidual.from_dict(value)
            elif field == "ideal" and not isinstance(value, Unset):
                value = ApiOutputEosPropertiesTPnResultIdeal.from_dict(value)

            kwargs[field] = value

        return cls(**kwargs)

    def __str__(self) -> str:
        """Formatted string representation of the `ApiOutputCalculationResultPhase` instance.
        Represents the properties using a table with columns: Property | Unit | Value.
        """
        def format_row(label: str, obj: Any) -> str:
            unit = getattr(obj, 'units', '-')
            value = getattr(obj, 'value', '-')

            if isinstance(value, float):
                value_str = f"{value:>15.8f}"
            else:
                value_str = f"{str(value):>15}"
            return f"{label:<25} | {unit:>15} | {value_str}"

        rows = ["TPn Result:"]
        rows.append(f"{'Property':<25} | {'Unit':>15} | {'Value':>15}")
        rows.append("-" * 62)

        rows.append(format_row("Temperature", self.temperature))
        rows.append(format_row("Pressure", self.pressure))
        rows.append(format_row("Volume", self.volume))
        rows.append(format_row("Attractive Pressure", self.attractive_pressure))
        rows.append(format_row("Repulsive Pressure", self.repulsive_pressure))

        if not isinstance(self.residual, Unset) and self.residual:
            rows.append("")
            rows.append("Residual Properties:")
            for attr_name in attr.fields_dict(type(self.residual)):
                val = getattr(self.residual, attr_name)
                rows.append(format_row(attr_name.replace("_", " ").title(), val))

        if not isinstance(self.ideal, Unset) and self.ideal:
            rows.append("")
            rows.append("Ideal Properties:")
            for attr_name in attr.fields_dict(type(self.ideal)):
                val = getattr(self.ideal, attr_name)
                rows.append(format_row(attr_name.replace("_", " ").title(), val))

        return "\n".join(rows)



"""
Module for representing a component composition of a fluid mixture.

This class is intended to serialize/deserialize data in a format compatible with the VLXE API.
"""
from typing import Any, Dict, List, Type, TypeVar, Union

import attr

from ..models.api_fluid_distribution_moment import ApiFluidDistributionMoment
from ..models.calculation_block_info import CalculationBlockInfo
from ..types import UNSET, Unset

T = TypeVar("T", bound="CalculationComposition")


@attr.s(auto_attribs=True)
class CalculationComposition:
    """Holds composition information for a component.
    
    Stores the compositional information for a component, which is to be stored in the `FlashCalculationInput.components` attribute.
    
    Attributes
    ----------
    component_id : str
        Id of the component.
    component_name : str
        Name of the component.
    mass : float
        Mass fraction of the component.
    sorting_order : int
        Index for sorting the components.
    moment : ApiFluidDistributionMoment
        Distribution moment information for the component.
    block_infos : List[CalculationBlockInfo]
        Block information for the component.
    """

    component_id: Union[Unset, str] = UNSET
    component_name: Union[Unset, None, str] = UNSET
    mass: Union[Unset, float] = UNSET
    sorting_order: Union[Unset, int] = UNSET
    moment: Union[Unset, ApiFluidDistributionMoment] = UNSET
    block_infos: Union[Unset, None, List[CalculationBlockInfo]] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        """Return a dictionary representation of this object."""
        component_id = self.component_id
        component_name = self.component_name
        mass = self.mass
        sorting_order = self.sorting_order
        moment: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.moment, Unset):
            moment = self.moment.to_dict()

        block_infos: Union[Unset, None, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.block_infos, Unset):
            if self.block_infos is None:
                block_infos = None
            else:
                block_infos = []
                for block_infos_item_data in self.block_infos:
                    block_infos_item = block_infos_item_data.to_dict()

                    block_infos.append(block_infos_item)

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if not isinstance(component_id, Unset):
            field_dict["componentId"] = component_id
        if not isinstance(component_name, Unset):
            field_dict["componentName"] = component_name
        if not isinstance(mass, Unset):
            field_dict["mass"] = mass
        if not isinstance(sorting_order, Unset):
            field_dict["sortingOrder"] = sorting_order
        if not isinstance(moment, Unset):
            field_dict["moment"] = moment
        if not isinstance(block_infos, Unset):
            field_dict["blockInfos"] = block_infos

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        """Create an instance of this class from a dict."""
        d = src_dict.copy()
        component_id = d.pop("componentId", UNSET)

        component_name = d.pop("componentName", UNSET)

        mass = d.pop("mass", UNSET)

        sorting_order = d.pop("sortingOrder", UNSET)

        _moment = d.pop("moment", UNSET)
        moment: Union[Unset, ApiFluidDistributionMoment]
        if isinstance(_moment, Unset):
            moment = UNSET
        else:
            moment = ApiFluidDistributionMoment.from_dict(_moment)

        block_infos = []
        _block_infos = d.pop("blockInfos", UNSET)
        for block_infos_item_data in _block_infos or []:
            block_infos_item = CalculationBlockInfo.from_dict(block_infos_item_data)

            block_infos.append(block_infos_item)

        calculation_composition = cls(
            component_id=component_id,
            component_name=component_name,
            mass=mass,
            sorting_order=sorting_order,
            moment=moment,
            block_infos=block_infos,
        )

        return calculation_composition

    def __str__(self) -> str:
        """
        Returns a string representation of the `CalculationComposition` instance.
        
        Includes:
        - Component name, ID, mass, sorting order, moment, and block infos.
        """
        parts = []
        name = self.component_name if not isinstance(self.component_name, Unset) else "N/A"
        comp_id = self.component_id if not isinstance(self.component_id, Unset) else "N/A"
        mass = self.mass if not isinstance(self.mass, Unset) else "N/A"
        order = self.sorting_order if not isinstance(self.sorting_order, Unset) else "N/A"
        
        parts.append(f"Component: {name} (ID: {comp_id})")
        parts.append(f"  Mass: {mass}")
        parts.append(f"  Sorting Order: {order}")

        # Moment
        if not isinstance(self.moment, Unset) and self.moment is not None:
            parts.append("  Moment:")
            parts.append("    " + str(self.moment).replace("\n", "\n    "))
        else:
            parts.append("  Moment: N/A")

        # Block Infos
        if not isinstance(self.block_infos, Unset) and self.block_infos:
            parts.append("  Block Infos:")
            for b in self.block_infos:
                parts.append("    " + str(b).replace("\n", "\n    "))
        else:
            parts.append("  Block Infos: N/A")

        return "\n".join(parts)



"""Module for representing standard fluid component metadata, including PC-SAFT parameters and thermodynamic constants.

This class is intended to serialize/deserialize data in a format compatible with the VLXE API.
"""
from typing import Any, Dict, Type, TypeVar, Union
import attr
from ..types import UNSET, Unset

T = TypeVar("T", bound="ApiFluidStandardComponent")


@attr.s(auto_attribs=True)
class ApiFluidStandardComponent:
    """Information for a standard fluid component.

    Stores the parameters for a *standard* component, which is to be stored in the `ApiFluid.standards` attribute.
    
    Attributes
    ----------
    formula : str
        Formula of the standard component.
    dippr_database_id : int
        Dippr database id of the standard component.
    name : str
        Name of the standard component.
    is_alkane : bool
        Whether the standard component is an alkane.
    sorting_order : int
        Index for sorting the standard components.
    molar_mass : float
        Molar mass (g/mol) of the standard component.
    critical_temperature : float
        Critical temperature (K) of the standard component.
    critical_pressure : float
        Critical pressure (bar) of the standard component.
    critical_volume : float
        Critical volume (cm^3/mol) of the standard component.
    acentric_factor : float
        Acentric factor (-) of the standard component.
    volume_shift : float
        Volume shift (cm3/g) of the standard component.
    pc_saft_epsilon : float
        PC-SAFT epsilon (K) of the standard component.
    pc_saft_sigma_0 : float
        PC-SAFT sigma_0 (Angstrom) of the standard component.
    pc_saft_sigma_1 : float
        PC-SAFT sigma_1 (Angstrom) of the standard component.
    pc_saft_sigma_2 : float
        PC-SAFT sigma_2 (1/K) of the standard component.
    pc_saft_sigma_3 : float
        PC-SAFT sigma_3 (Angstrom) of the standard component.
    pc_saft_sigma_4 : float
        PC-SAFT sigma_4 (1/K) of the standard component.
    pc_saftdm : float
        PC-SAFT dm (mol/g) of the standard component.
    pc_saft_ab_active : bool
        PC-SAFT association active of the standard component.
    pc_saft_ab_kappa : float
        PC-SAFT association bond volume (-) of the standard component.
    pc_saft_ab_epsilon : float
        PC-SAFT association energy (K) of the standard component.
    pc_saft_ab_scheme : str
        PC-SAFT association scheme (-) of the standard component.
    pc_saft_polar_active : bool
        PC-SAFT polar active of the standard component.
    pc_saft_polarx : float
        PC-SAFT polar x (-) of the standard component.
    pc_saft_polar_d : float
        PC-SAFT polar dipole strength (D) of the standard component.
    pc_saft_cp_ig_poly_c[i] : float
        PC-SAFT ideal gas polynomial coefficients for heat capacity.
    pc_saft_cp_ig_dippr_c[i] : float
        PC-SAFT ideal gas DIPPR coefficients for heat capacity.
    ideal_gas_enthalpy_of_formation : float
        Ideal gas Enthalpy of formation (kJ/kg) of the standard component.
    ideal_gas_gibbs_energy_of_formation : float
        Ideal gas Gibbs energy of formation (kJ/kg) of the standard component.
    ideal_gas_absolute_entropy : float
        Ideal gas absolute entropy (kJ/kg) of the standard component.
    standard_state_enthalpy_of_formation : float
        Standard state Enthalpy of formation (kJ/kg) of the standard component.
    standard_state_gibbs_energy_of_formation : float
        Standard state Gibbs energy of formation (kJ/kg) of the standard component.
    standard_state_absolute_entropy : float
        Standard state absolute entropy (kJ/kg) of the standard component.
    """

    formula: Union[Unset, None, str] = UNSET
    dippr_database_id: Union[Unset, int] = UNSET
    name: Union[Unset, None, str] = UNSET
    is_alkane: Union[Unset, bool] = UNSET
    sorting_order: Union[Unset, int] = UNSET
    molar_mass: Union[Unset, float] = UNSET
    critical_temperature: Union[Unset, float] = UNSET
    critical_pressure: Union[Unset, float] = UNSET
    critical_volume: Union[Unset, float] = UNSET
    acentric_factor: Union[Unset, float] = UNSET
    volume_shift: Union[Unset, float] = UNSET
    pc_saft_epsilon: Union[Unset, float] = UNSET
    pc_saft_sigma_0: Union[Unset, float] = UNSET
    pc_saft_sigma_1: Union[Unset, float] = UNSET
    pc_saft_sigma_2: Union[Unset, float] = UNSET
    pc_saft_sigma_3: Union[Unset, float] = UNSET
    pc_saft_sigma_4: Union[Unset, float] = UNSET
    pc_saftdm: Union[Unset, float] = UNSET
    pc_saft_ab_active: Union[Unset, bool] = UNSET
    pc_saft_ab_kappa: Union[Unset, float] = UNSET
    pc_saft_ab_epsilon: Union[Unset, float] = UNSET
    pc_saft_ab_scheme: Union[Unset, str] = UNSET
    pc_saft_polar_active: Union[Unset, bool] = UNSET
    pc_saft_polarx: Union[Unset, float] = UNSET
    pc_saft_polar_d: Union[Unset, float] = UNSET
    pc_saft_cp_ig_poly_c0: Union[Unset, float] = UNSET
    pc_saft_cp_ig_poly_c1: Union[Unset, float] = UNSET
    pc_saft_cp_ig_poly_c2: Union[Unset, float] = UNSET
    pc_saft_cp_ig_poly_c3: Union[Unset, float] = UNSET
    pc_saft_cp_ig_poly_c4: Union[Unset, float] = UNSET
    pc_saft_cp_ig_poly_c5: Union[Unset, float] = UNSET
    pc_saft_cp_ig_poly_c6: Union[Unset, float] = UNSET
    pc_saft_cp_ig_poly_c7: Union[Unset, float] = UNSET
    pc_saft_cp_ig_dippr_c0: Union[Unset, float] = UNSET
    pc_saft_cp_ig_dippr_c1: Union[Unset, float] = UNSET
    pc_saft_cp_ig_dippr_c2: Union[Unset, float] = UNSET
    pc_saft_cp_ig_dippr_c3: Union[Unset, float] = UNSET
    pc_saft_cp_ig_dippr_c4: Union[Unset, float] = UNSET
    pc_saft_cp_ig_dippr_c5: Union[Unset, float] = UNSET
    pc_saft_cp_ig_dippr_c6: Union[Unset, float] = UNSET
    ideal_gas_enthalpy_of_formation: Union[Unset, float] = UNSET
    ideal_gas_gibbs_energy_of_formation: Union[Unset, float] = UNSET
    ideal_gas_absolute_entropy: Union[Unset, float] = UNSET
    standard_state_enthalpy_of_formation: Union[Unset, float] = UNSET
    standard_state_gibbs_energy_of_formation: Union[Unset, float] = UNSET
    standard_state_absolute_entropy: Union[Unset, float] = UNSET

    # Mapping from attribute name to VLXE API key
    FIELD_MAP: Dict[str, str] = {
        # general component data
        "formula": "formula",
        "dippr_database_id": "dipprDatabaseId",
        "name": "name",
        "is_alkane": "isAlkane",
        "sorting_order": "sortingOrder",
        "molar_mass": "molarMass",
        "critical_temperature": "criticalTemperature",
        "critical_pressure": "criticalPressure",
        "critical_volume": "criticalVolume",
        "acentric_factor": "acentricFactor",
        # model parameter data
        "volume_shift": "volumeShift",
        "pc_saft_epsilon": "pcSaftEpsilon",
        "pc_saft_sigma_0": "pcSaftSigma0",
        "pc_saft_sigma_1": "pcSaftSigma1",
        "pc_saft_sigma_2": "pcSaftSigma2",
        "pc_saft_sigma_3": "pcSaftSigma3",
        "pc_saft_sigma_4": "pcSaftSigma4",
        "pc_saftdm": "pcSaftdm",
        "pc_saft_ab_active": "pcSaftAbActive",
        "pc_saft_ab_kappa": "pcSaftAbKappa",
        "pc_saft_ab_epsilon": "pcSaftAbEpsilon",
        "pc_saft_ab_scheme": "pcSaftAbScheme",
        "pc_saft_polar_active": "pcSaftPolarActive",
        "pc_saft_polarx": "pcSaftPolarx",
        "pc_saft_polar_d": "pcSaftPolarD",
        "pc_saft_cp_ig_poly_c0": "pcSaftCpIgPolyC0",
        "pc_saft_cp_ig_poly_c1": "pcSaftCpIgPolyC1",
        "pc_saft_cp_ig_poly_c2": "pcSaftCpIgPolyC2",
        "pc_saft_cp_ig_poly_c3": "pcSaftCpIgPolyC3",
        "pc_saft_cp_ig_poly_c4": "pcSaftCpIgPolyC4",
        "pc_saft_cp_ig_poly_c5": "pcSaftCpIgPolyC5",
        "pc_saft_cp_ig_poly_c6": "pcSaftCpIgPolyC6",
        "pc_saft_cp_ig_poly_c7": "pcSaftCpIgPolyC7",
        "pc_saft_cp_ig_dippr_c0": "pcSaftCpIgDipprC0",
        "pc_saft_cp_ig_dippr_c1": "pcSaftCpIgDipprC1",
        "pc_saft_cp_ig_dippr_c2": "pcSaftCpIgDipprC2",
        "pc_saft_cp_ig_dippr_c3": "pcSaftCpIgDipprC3",
        "pc_saft_cp_ig_dippr_c4": "pcSaftCpIgDipprC4",
        "pc_saft_cp_ig_dippr_c5": "pcSaftCpIgDipprC5",
        "pc_saft_cp_ig_dippr_c6": "pcSaftCpIgDipprC6",
        # thermodynamic constants
        "ideal_gas_enthalpy_of_formation": "idealGasEnthalpyOfFormation",
        "ideal_gas_gibbs_energy_of_formation": "idealGasGibbsEnergyOfFormation",
        "ideal_gas_absolute_entropy": "idealGasAbsoluteEntropy",
        "standard_state_enthalpy_of_formation": "standardStateEnthalpyOfFormation",
        "standard_state_gibbs_energy_of_formation": "standardStateGibbsEnergyOfFormation",
        "standard_state_absolute_entropy": "standardStateAbsoluteEntropy",
    }

    def to_dict(self) -> Dict[str, Any]:
        """Dump an instance of `ApiFluidStandardComponent` to a dict."""
        result: Dict[str, Any] = {}
        for attr_name, json_key in self.FIELD_MAP.items():
            val = getattr(self, attr_name)
            if not isinstance(val, Unset):
                result[json_key] = val
        return result

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        """Create an instance of `ApiFluidStandardComponent` from a dict."""
        reverse_map = {v: k for k, v in cls.FIELD_MAP.items()}
        kwargs = {
            attr_name: src_dict.get(json_key, UNSET)
            for json_key, attr_name in reverse_map.items()
        }
        return cls(**kwargs)
    
    def __str__(self) -> str:
        """
        Returns a compact string representation of the `ApiFluidStandardComponent` instance.
        
        Includes only attributes that are not `Unset`.
        """
        items = []
        for attr_name, json_key in self.FIELD_MAP.items():
            val = getattr(self, attr_name)
            if not isinstance(val, Unset):
                items.append(f"{attr_name}={val}")
        return ", ".join(items)



from equia.models import (
  ApiFluid, ApiFluidStandardComponent, 
  ApiFluidKij
)

def demofluid_Methane_nHexane_Toluene(is_volume_shift=True) -> ApiFluid:
    """ Predefined fluid with 3 solvents using PR78 with volume shift."""   
    fluid = ApiFluid()
    fluid.name = "Methane + n-Hexane + Toluene"
    fluid.eos = "PR78"
    fluid.solvent_cp = "DIPPR"
    fluid.polymer_cp = "Polynomial"

    fluid.standards = [
      ApiFluidStandardComponent(name="Methane", is_alkane=True, molar_mass=16.04246,         
        acentric_factor = 0.0115478,
        critical_pressure = 45.99,
        critical_temperature = 190.564,
        critical_volume = 98.6278,
        volume_shift=-0.21013 if is_volume_shift else 0,
        pc_saftdm = 0.06233458,
        pc_saft_sigma_0 = 3.7039,
        pc_saft_epsilon = 150.03,
        ideal_gas_enthalpy_of_formation = -4645.16129,
        pc_saft_cp_ig_dippr_c0 = 2.075611657,
        pc_saft_cp_ig_dippr_c1 = 4.982577528,
        pc_saft_cp_ig_dippr_c2 = 2086.9,
        pc_saft_cp_ig_dippr_c3 = 2.593236715,
        pc_saft_cp_ig_dippr_c4 = 991.96,
        pc_saft_cp_ig_dippr_c5 = 50,
        pc_saft_cp_ig_dippr_c6 = 1500
      ),
      ApiFluidStandardComponent(name="n-Hexane", is_alkane=True, molar_mass=86.17536,         
        acentric_factor = 0.301261,
        critical_pressure = 30.25,
        critical_temperature = 507.6,
        critical_volume = 369.566,
        volume_shift = 0.011898058 if is_volume_shift else 0,
        pc_saftdm = 0.03548,
        pc_saft_sigma_0 = 3.7983,
        pc_saft_epsilon = 236.77,
        ideal_gas_enthalpy_of_formation = -1937.212679,
        pc_saft_cp_ig_dippr_c0 = 1.21148319,
        pc_saft_cp_ig_dippr_c1 = 4.088175553,
        pc_saft_cp_ig_dippr_c2 = 1694.6,
        pc_saft_cp_ig_dippr_c3 = 2.749045667,
        pc_saft_cp_ig_dippr_c4 = 761.6,
        pc_saft_cp_ig_dippr_c5 = 200,
        pc_saft_cp_ig_dippr_c6 = 1500
      ),
      
      ApiFluidStandardComponent(name="Toluene", is_alkane=False, molar_mass=92.141,         
        acentric_factor = 0.2621,
        critical_pressure = 41.05992975,
        critical_temperature = 591.8,
        critical_volume = 316.0,
        volume_shift=0.065471176 if is_volume_shift else 0,
        pc_saftdm = 0.030549918,
        pc_saft_sigma_0 = 3.7169,
        pc_saft_epsilon = 285.69,
        ideal_gas_enthalpy_of_formation = -1753.282643,
        pc_saft_cp_ig_dippr_c0 = 1.175133721,
        pc_saft_cp_ig_dippr_c1 = 3.762255267,
        pc_saft_cp_ig_dippr_c2 = 1614.1,
        pc_saft_cp_ig_dippr_c3 = 2.658107495,
        pc_saft_cp_ig_dippr_c4 = 742,
        pc_saft_cp_ig_dippr_c5 = 200,
        pc_saft_cp_ig_dippr_c6 = 1500
      )
    ]
    
    fluid.kij = [
      ApiFluidKij(index_i=0, index_j=1, kija=0.0),
      ApiFluidKij(index_i=0, index_j=2, kija=0.0),
      ApiFluidKij(index_i=1, index_j=2, kija=0.0),
      ]

    return fluid



import datetime
from typing import Any, Dict, List, Type, TypeVar, Union

import attr
from dateutil.parser import isoparse

from ..models.api_fluid_kij import ApiFluidKij
from ..models.api_fluid_polymer_component import ApiFluidPolymerComponent
from ..models.api_fluid_standard_component import ApiFluidStandardComponent
from ..types import UNSET, Unset

T = TypeVar("T", bound="ApiFluid")


@attr.s(auto_attribs=True)
class ApiFluid:
    """ Information for a fluid.
    To be used in the `FlashCalculationInput.fluid` attribute.
    
    Attributes
    ----------
    fluidid : str
        Unique identifier for the fluid.
    creation_time : datetime.datetime
        Date and time the fluid was created.
    name : str
        Name of the fluid.
    comment : str
        Comment for the fluid.
    eos : str
        Equation of state for the fluid. 
        Options: `{'SRK', 'PR', 'PR78', 'PC-SAFT'}`
    property_reference_point : str
        Reference point for the fluid.
    solvent_cp : str
        Heat capacity model for the solvents.
    polymer_cp : str
        Heat capacity model for the polymers.
    standards : List[ApiFluidStandardComponent]
        Model parameters for each standard component.
    polymers : List[ApiFluidPolymerComponent]
        Model parameters for each polymer component.
    kij : List[ApiFluidKij]
        kij (or BIP, binary interactions parameters) for the fluid.
    """

    fluidid: Union[Unset, str] = UNSET
    creation_time: Union[Unset, datetime.datetime] = UNSET
    name: Union[Unset, None, str] = UNSET
    comment: Union[Unset, None, str] = UNSET
    eos: Union[Unset, str] = UNSET
    property_reference_point: Union[Unset, str] = UNSET
    solvent_cp: Union[Unset, str] = UNSET
    polymer_cp: Union[Unset, str] = UNSET
    standards: Union[Unset, None, List[ApiFluidStandardComponent]] = UNSET
    polymers: Union[Unset, None, List[ApiFluidPolymerComponent]] = UNSET
    kij: Union[Unset, None, List[ApiFluidKij]] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        """Returns the `ApiFluid` attributes as a dict."""
        fluidid = self.fluidid
        creation_time: Union[Unset, str] = UNSET
        if not isinstance(self.creation_time, Unset):
            creation_time = self.creation_time.isoformat()

        name = self.name
        comment = self.comment
        eos = self.eos

        property_reference_point = self.property_reference_point

        solvent_cp = self.solvent_cp

        polymer_cp = self.polymer_cp

        standards: Union[Unset, None, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.standards, Unset):
            if self.standards is None:
                standards = None
            else:
                standards = []
                for standards_item_data in self.standards:
                    standards_item = standards_item_data.to_dict()

                    standards.append(standards_item)

        polymers: Union[Unset, None, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.polymers, Unset):
            if self.polymers is None:
                polymers = None
            else:
                polymers = []
                for polymers_item_data in self.polymers:
                    polymers_item = polymers_item_data.to_dict()

                    polymers.append(polymers_item)

        kij: Union[Unset, None, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.kij, Unset):
            if self.kij is None:
                kij = None
            else:
                kij = []
                for kij_item_data in self.kij:
                    kij_item = kij_item_data.to_dict()

                    kij.append(kij_item)

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if not isinstance(fluidid, Unset):
            field_dict["fluidId"] = fluidid
        if not isinstance(creation_time, Unset):
            field_dict["creationTime"] = creation_time
        if not isinstance(name, Unset):
            field_dict["name"] = name
        if not isinstance(comment, Unset):
            field_dict["comment"] = comment
        if not isinstance(eos, Unset):
            field_dict["eos"] = eos
        if not isinstance(property_reference_point, Unset):
            field_dict["propertyReferencePoint"] = property_reference_point
        if not isinstance(solvent_cp, Unset):
            field_dict["solventCp"] = solvent_cp
        if not isinstance(polymer_cp, Unset):
            field_dict["polymerCp"] = polymer_cp
        if not isinstance(standards, Unset):
            field_dict["standards"] = standards
        if not isinstance(polymers, Unset):
            field_dict["polymers"] = polymers
        if not isinstance(kij, Unset):
            field_dict["kij"] = kij

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        """Create an instance of `ApiFluid` from a dict representation."""
        d = src_dict.copy()
        fluidid = d.pop("fluidId", UNSET)

        _creation_time = d.pop("creationTime", UNSET)
        creation_time: Union[Unset, datetime.datetime]
        if isinstance(_creation_time, Unset):
            creation_time = UNSET
        else:
            creation_time = isoparse(_creation_time)

        name = d.pop("name", UNSET)

        comment = d.pop("comment", UNSET)

        eos = d.pop("eos", UNSET)

        property_reference_point = d.pop("propertyReferencePoint", UNSET)

        solvent_cp = d.pop("solventCp", UNSET)

        polymer_cp = d.pop("polymerCp", UNSET)

        standards = []
        _standards = d.pop("standards", UNSET)
        for standards_item_data in _standards or []:
            standards_item = ApiFluidStandardComponent.from_dict(standards_item_data)

            standards.append(standards_item)

        polymers = []
        _polymers = d.pop("polymers", UNSET)
        for polymers_item_data in _polymers or []:
            polymers_item = ApiFluidPolymerComponent.from_dict(polymers_item_data)

            polymers.append(polymers_item)

        kij = []
        _kij = d.pop("kij", UNSET)
        for kij_item_data in _kij or []:
            kij_item = ApiFluidKij.from_dict(kij_item_data)

            kij.append(kij_item)

        api_fluid = cls(
            fluidid=fluidid,
            creation_time=creation_time,
            name=name,
            comment=comment,
            eos=eos,
            property_reference_point=property_reference_point,
            solvent_cp=solvent_cp,
            polymer_cp=polymer_cp,
            standards=standards,
            polymers=polymers,
            kij=kij,
        )

        return api_fluid

    def __str__(self) -> str:
        """
        Returns a string representation of the `ApiFluid` instance.

        Includes basic fluid metadata and a listing of standard components.
        """
        parts = []
        name = self.name if not isinstance(self.name, Unset) else "N/A"
        fluidid = self.fluidid if not isinstance(self.fluidid, Unset) else "N/A"
        created = (
            self.creation_time.isoformat()
            if not isinstance(self.creation_time, Unset)
            else "N/A"
        )

        parts.append(f"Fluid: {name} (ID: {fluidid})")
        parts.append(f"  Created: {created}")
        parts.append(f"  Comment: {self.comment if not isinstance(self.comment, Unset) else "N/A"}")
        parts.append(f"  EOS: {self.eos if not isinstance(self.eos, Unset) else "N/A"}")
        parts.append(f"  Property Reference Point: {self.property_reference_point if not isinstance(self.property_reference_point, Unset) else 'N/A'}")
        parts.append(f"  Solvent Cp: {self.solvent_cp if not isinstance(self.solvent_cp, Unset) else 'N/A'}")
        parts.append(f"  Polymer Cp: {self.polymer_cp if not isinstance(self.polymer_cp, Unset) else 'N/A'}")

        # Standard components
        if not isinstance(self.standards, Unset) and self.standards:
            parts.append("  Standard Components:")
            for s in self.standards:
                parts.append(f"    - {s}")
        else:
            parts.append("  Standard Components: None")

        # Polymer components
        if not isinstance(self.polymers, Unset) and self.polymers:
            parts.append("  Polymer Components:")
            for p in self.polymers:
                parts.append(f"    - {p}")
        else:
            parts.append("  Polymer Components: None")

        # Kij
        if not isinstance(self.kij, Unset) and self.kij:
            parts.append("  Kij:")
            for k in self.kij:
                parts.append(f"    - {k}")
        else:
            parts.append("  Kij: None")

        return "\n".join(parts)



"""
Module for representing a flash calculation input.
"""
from typing import Any, Dict, List, Type, TypeVar, Union

import attr

from ..models.calculation_composition import CalculationComposition
from ..models.api_fluid import ApiFluid
from ..types import UNSET, Unset

T = TypeVar("T", bound="FlashCalculationInput")


@attr.s(auto_attribs=True)
class FlashCalculationInput:
    """Input for flash calculation.

    Attributes
    ----------
    access_key : str
        Access key for the API.
    components : List[CalculationComposition] 
        Component composition
    flashtype : str 
        Type of flash to perform. 
        Allowed options: `{'Fixed Temperature/Pressure', 'Fixed Pressure/Enthalpy', 'Fixed Pressure/Entropy'}`
    units : str 
        Units used for input and output.
    fluidid : str 
        Id of fluid on webserver. Must be defined if `fluid` argument is not given.
    fluid : ApiFluid
        Fluid information
    temperature : float
        Temperature in units given in `units` argument.
    pressure : float
        Pressure in units given in `units` argument.
    enthalpy : float
        Enthalpy in units given in `units` argument.
    entropy : float
        Entropy in units given in `units` argument.
    """
    access_key: str
    components: List[CalculationComposition] # Component composition
    flashtype: str # Type of flash to perform. Allowed option: Fixed Temperature/Pressure, Fixed Pressure/Enthalpy or Fixed Pressure/Entropy
    units: str # Units used for input and output
    fluidid: Union[Unset, str] = UNSET #Id of fluid on webserver. Must be defined if no fluid given in fluid argument
    fluid: Union[Unset, ApiFluid] = UNSET #Fluid information
    temperature: Union[Unset, float] = UNSET #Temperature in units given in 'Units' argument
    pressure: Union[Unset, float] = UNSET #Pressure in units given in 'Units' argument
    enthalpy: Union[Unset, float] = UNSET #Enthalpy in units given in 'Units' argument
    entropy: Union[Unset, float] = UNSET #Entropy in units given in 'Units' argument

    def to_dict(self) -> Dict[str, Any]:
        """Return a dictionary representation of this object."""
        access_key = self.access_key
        components = []
        for components_item_data in self.components:
            components_item = components_item_data.to_dict()
            components.append(components_item)

        flashtype = self.flashtype
        units = self.units
        temperature = self.temperature
        pressure = self.pressure
        enthalpy = self.enthalpy
        entropy = self.entropy

        fluidid: Union[Unset, str] = UNSET
        if not isinstance(self.fluidid, Unset):
            fluid = self.fluidid

        fluid: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.fluid, Unset):
            fluid = self.fluid.to_dict()

        field_dict: Dict[str, Any] = {}
        field_dict.update(
            {
                "accessKey": access_key,
                "components": components,
                "flashType": flashtype,
            }
        )
        if not isinstance(units, Unset):
            field_dict["units"] = units
        if not isinstance(fluidid, Unset):
            field_dict["fluidId"] = fluidid
        if not isinstance(fluid, Unset):
            field_dict["fluid"] = fluid
        if not isinstance(temperature, Unset):
            field_dict["temperature"] = temperature
        if not isinstance(pressure, Unset):
            field_dict["pressure"] = pressure
        if not isinstance(enthalpy, Unset):
            field_dict["enthalpy"] = enthalpy
        if not isinstance(entropy, Unset):
            field_dict["entropy"] = entropy

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        """Create an instance of `FlashCalculationInput` from a dict representation."""
        d = src_dict.copy()
        access_key = d.pop("accessKey")
        flashtype = d.pop("flashType")
        units = d.pop("units", UNSET)

        components = []
        _components = d.pop("components")
        for components_item_data in _components:
            components_item = CalculationComposition.from_dict(
                components_item_data)

            components.append(components_item)

        _fluidid = d.pop("fluidId", UNSET)
        fluidid: Union[Unset, str]
        if isinstance(_fluidid, Unset):
            fluidid = UNSET
        else:
            fluidid = _fluidid

        _fluid = d.pop("fluid", UNSET)
        fluid: Union[Unset, ApiFluid]
        if isinstance(_fluid, Unset):
            fluid = UNSET
        else:
            fluid = ApiFluid.from_dict(_fluid)

        temperature = d.pop("temperature", UNSET)
        pressure = d.pop("pressure", UNSET)
        enthalpy = d.pop("enthalpy", UNSET)
        entropy = d.pop("entropy", UNSET)

        flash_calculation_input = cls(
            access_key=access_key,
            components=components,
            flashtype=flashtype,
            units=units,
            fluidid=fluidid,
            fluid=fluid,
            temperature=temperature,
            pressure=pressure,
            enthalpy=enthalpy,
            entropy=entropy,
        )

        return flash_calculation_input

    def __str__(self) -> str:
        """
        Returns a string representation of the `FlashCalculationInput` instance.
        
        Includes:
        - Flash type, units, specifications, fluid composition and parameters.
        """
        parts = ["\n"]
        parts.append("FlashCalculationInput:")
        
        # Flash Info
        parts.append(f"  Flash Type: {self.flashtype}")
        parts.append(f"  Units: {self.units}")
        parts.append(f"  Fluid ID: {self.fluidid if not isinstance(self.fluidid, Unset) else 'N/A'}")
        
        # Specifications
        def fmt_val(val, label):
            return f"  {label}: {val}" if not isinstance(val, Unset) else f"  {label}: N/A"

        parts.append(fmt_val(self.temperature, "Temperature"))
        parts.append(fmt_val(self.pressure, "Pressure"))
        parts.append(fmt_val(self.enthalpy, "Enthalpy"))
        parts.append(fmt_val(self.entropy, "Entropy"))

        # Composition
        if self.components:
            parts.append("  Components:")
            for c in self.components:
                parts.append("    " + str(c).replace("\n", "\n    "))
        else:
            parts.append("  Components: N/A")

        # Fluid
        if not isinstance(self.fluid, Unset) and self.fluid is not None:
            parts.append("  Fluid:")
            parts.append("    " + str(self.fluid).replace("\n", "\n    "))
        else:
            parts.append("  Fluid: N/A")

        return "\n".join(parts)
